@@ -1,6 +1,6 @@
 /* Copyright (C) 2003 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: listcontrol.c,v 1.35 2005/02/03 14:53:15 mmj Exp $
+ * $Id: listcontrol.c,v 1.40 2005/05/02 17:39:10 mmj Exp $
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -36,12 +36,14 @@
 #include "find_email_adr.h"
 #include "strgen.h"
 #include "send_help.h"
+#include "send_list.h"
 #include "log_error.h"
 #include "statctrl.h"
 #include "mygetline.h"
 #include "chomp.h"
 #include "memory.h"
 #include "log_oper.h"
+#include "ctrlvalues.h"
 
 enum ctrl_e {
 	CTRL_SUBSCRIBE_DIGEST,
@@ -60,6 +62,7 @@ enum ctrl_e {
 	CTRL_MODERATE,
 	CTRL_HELP,
 	CTRL_GET,
+	CTRL_LIST,
 	CTRL_END  /* end marker, must be last */
 };
 
@@ -87,7 +90,8 @@ static struct ctrl_command ctrl_commands[] = {
 	{ "bounces",            1 },
 	{ "moderate",           1 },
 	{ "help",               0 },
-	{ "get",                1 }
+	{ "get",                1 },
+	{ "list",               0 }
 };
 
 
@@ -97,14 +101,15 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 		const char *mlmmjbounce, const char *mailname)
 {
 	char *atsign, *recipdelimsign, *bouncenr, *tmpstr;
-	char *controlstr, *param, *conffilename, *moderatefilename;
-	char *c, *archivefilename;
+	char *controlstr, *param = NULL, *conffilename, *moderatefilename;
+	char *c, *archivefilename, *sendfilename;
 	const char *subswitch;
 	size_t len;
 	struct stat stbuf;
-	int closedlist, nosubconfirm, tmpfd, noget;
+	int closedlist, nosubconfirm, tmpfd, noget, i;
 	size_t cmdlen;
 	unsigned int ctrl;
+	struct strlist *owners;
 	
 	/* A closed list doesn't allow subscribtion and unsubscription */
 	closedlist = statctrl(listdir, "closedlist");
@@ -160,6 +165,14 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 
 		}
 	}
+	
+	/* Only allow mails with bad From: header to be bounce mails */
+	if(fromemails->emailcount != 1 && ctrl != CTRL_BOUNCES) {
+		log_error(LOG_ARGS, "Discarding mail with invalid From: "
+				"which was not a bounce");
+		unlink(mailname);
+		exit(EXIT_SUCCESS);
+	}
 
 	switch (ctrl) {
 
@@ -478,16 +491,25 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 		/* TODO Add accept/reject parameter to moderate */
 		unlink(mailname);
 		moderatefilename = concatstr(3, listdir, "/moderation/", param);
+		sendfilename = concatstr(2, moderatefilename, ".sending");
 		myfree(param);
+
 		if(stat(moderatefilename, &stbuf) < 0) {
 			myfree(moderatefilename);
 			exit(EXIT_SUCCESS); /* just exit, no mail to moderate */
 		}
+		/* Rename it to avoid mail being sent twice */
+		if(rename(moderatefilename, sendfilename) < 0) {
+			log_error(LOG_ARGS, "Could not rename to .sending");
+			exit(EXIT_FAILURE);
+		}
+
 		log_oper(listdir, OPLOGFNAME, "%s moderated %s",
 				fromemails->emaillist[0], moderatefilename);
+		myfree(moderatefilename);
 		execlp(mlmmjsend, mlmmjsend,
 				"-L", listdir,
-				"-m", moderatefilename, (char *)NULL);
+				"-m", sendfilename, (char *)NULL);
 		log_error(LOG_ARGS, "execlp() of '%s' failed",
 					mlmmjsend);
 		exit(EXIT_FAILURE);
@@ -531,6 +553,21 @@ int listcontrol(struct email_container *fromemails, const char *listdir,
 					mlmmjsend);
 		exit(EXIT_FAILURE);
 		break;
+
+	/* listname+list@domain.tld */
+	case CTRL_LIST:
+		unlink(mailname);
+		owners = ctrlvalues(listdir, "owner");
+		for(i = 0; i < owners->count; i++) {
+			if(strcmp(fromemails->emaillist[0],
+						owners->strs[i]) == 0) {
+				log_oper(listdir, OPLOGFNAME,
+						"%s requested sub list",
+				fromemails->emaillist[0]);
+				send_list(listdir, owners->strs[i], mlmmjsend);
+			}
+		}
+		break;
 	}
 
 	unlink(mailname);
