@@ -1,6 +1,6 @@
 /* Copyright (C) 2004 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: mlmmj-maintd.c,v 1.51 2005/02/14 09:32:02 mmj Exp $
+ * $Id: mlmmj-maintd.c,v 1.53 2005/05/03 07:50:31 mmj Exp $
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -85,7 +85,8 @@ static int mydaemon(const char *rootdir)
 	else if (pid)
 		exit(EXIT_SUCCESS); /* parent says bye bye */
 
-	chdir(rootdir);
+	if(chdir(rootdir) < 0)
+		log_error(LOG_ARGS, "Could not chdir(%s)", rootdir);
 
 	i = sysconf(_SC_OPEN_MAX);
 	if(i < 0)
@@ -208,9 +209,11 @@ int resend_queue(const char *listdir, const char *mlmmjsend)
 	struct dirent *dp;
 	char *mailname, *fromname, *toname, *reptoname, *from, *to, *repto;
 	char *ch, *dirname = concatstr(2, listdir, "/queue/");
+	char *bouncelifestr;
 	pid_t childpid, pid;
 	struct stat st;
 	int fromfd, tofd, fd, discarded = 0, status;
+	time_t t, bouncelife = 0;
 
 	if(chdir(dirname) < 0) {
 		log_error(LOG_ARGS, "Could not chdir(%s)", dirname);
@@ -291,6 +294,21 @@ int resend_queue(const char *listdir, const char *mlmmjsend)
 			myfree(reptoname);
 		}
 
+		/* before we try again, check and see if it's old */
+		bouncelifestr = ctrlvalue(listdir, "bouncelife");
+		if(bouncelifestr) {
+			bouncelife = atol(bouncelifestr);
+			myfree(bouncelifestr);
+		}
+		if(bouncelife == 0)
+			bouncelife = BOUNCELIFE;
+
+		t = time(NULL);
+		if(t - st.st_mtime > bouncelife) {
+			unlink(mailname);
+			continue;
+		}
+
 		childpid = fork();
 
 		if(childpid < 0) {
