@@ -1,6 +1,6 @@
-/* Copyright (C) 2002, 2003, 2004 Mads Martin Joergensen <mmj at mmj.dk>
+/* Copyright (C) 2004, 2003, 2004 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: mlmmj-send.c,v 1.76 2005/02/14 22:56:45 mmj Exp $
+ * $Id: mlmmj-send.c,v 1.83 2005/05/04 09:37:54 mmj Exp $
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -41,6 +41,7 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <signal.h>
 
 #include "mlmmj.h"
 #include "mlmmj-send.h"
@@ -64,6 +65,12 @@
 static int addtohdr = 0;
 static int prepmailinmem = 0;
 static int maxverprecips = MAXVERPRECIPS;
+static int gotsigterm = 0;
+
+void catch_sig_term(int sig)
+{
+	gotsigterm = 1;
+}
 
 char *bounce_from_adr(const char *recipient, const char *listadr,
 		      const char *mailfilename)
@@ -178,6 +185,13 @@ int send_mail(int sockfd, const char *from, const char *to,
 {
 	int retval = 0;
 	char *reply, *tohdr;
+
+	if(strchr(to, '@') == NULL) {
+		errno = 0;
+		log_error(LOG_ARGS, "No @ in address, ignoring %s",
+				to);
+		return 0;
+	}
 	
 	retval = write_mail_from(sockfd, from, "");
 	if(retval) {
@@ -367,6 +381,17 @@ int send_mail_verp(int sockfd, struct strlist *addrs, char *mailmap,
 		return MLMMJ_FROM;
 	}
 	for(i = 0; i < addrs->count; i++) {
+		if(gotsigterm) {
+			log_error(LOG_ARGS, "TERM signal recieved, "
+					"shutting down.");
+			return -1;
+		}
+		if(strchr(addrs->strs[i], '@') == NULL) {
+			errno = 0;
+			log_error(LOG_ARGS, "No @ in address, ignoring %s",
+					addrs->strs[i]);
+			continue;
+		}
 		retval = write_rcpt_to(sockfd, addrs->strs[i]);
 		if(retval) {
 			log_error(LOG_ARGS, "Could not write RCPT TO:\n");
@@ -542,6 +567,19 @@ int send_mail_many_list(int sockfd, const char *from, const char *replyto,
 
 	for(i = 0; i < addrs->count; i++) {
 		addr = addrs->strs[i];
+		if(strchr(addr, '@') == NULL) {
+			errno = 0;
+			log_error(LOG_ARGS, "No @ in address, ignoring %s",
+					addr);
+			continue;
+		}
+		if(gotsigterm && listaddr && archivefilename) {
+			/* we got SIGTERM, so save the addresses and bail */
+			log_error(LOG_ARGS, "TERM signal recieved, "
+						"shutting down.");
+			index = mybasename(archivefilename);
+			return requeuemail(listdir, index, addrs, i);
+		}
 		if(from) {
 			res = send_mail(sockfd, from, addr, replyto,
 					    mailmap, mailsize, listdir, NULL,
@@ -612,6 +650,7 @@ int main(int argc, char **argv)
 	uid_t uid;
 	struct strlist stl;
 	unsigned short smtpport = 25;
+	struct sigaction sigact;
 
 	CHECKFULLPATH(argv[0]);
 	
@@ -621,6 +660,13 @@ int main(int argc, char **argv)
 	mlmmjbounce = concatstr(2, bindir, "/mlmmj-bounce");
 	myfree(bindir);
 	
+	/* install signal handler for SIGTERM */
+	sigact.sa_handler = catch_sig_term;
+	sigemptyset(&sigact.sa_mask);
+	sigact.sa_flags = 0;
+	if(sigaction(SIGTERM, &sigact, NULL) < 0)
+		log_error(LOG_ARGS, "Could not install SIGTERM handler!");
+
 	while ((opt = getopt(argc, argv, "aVDhm:l:L:R:F:T:r:s:")) != -1){
 		switch(opt) {
 		case 'a':
