@@ -1,6 +1,6 @@
 /* Copyright (C) 2002, 2003 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: mlmmj-sub.c,v 1.38 2005/02/14 22:56:45 mmj Exp $
+ * $Id: mlmmj-sub.c,v 1.41 2005/05/12 07:19:51 mmj Exp $
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -240,6 +240,35 @@ static void print_help(const char *prg)
 	exit(EXIT_SUCCESS);
 }
 
+void generate_subscribed(const char *listdir, const char *subaddr,
+		const char *mlmmjsend)
+{
+	char *queuefilename, *fromaddr, *listname, *listfqdn, *listaddr;
+
+	listaddr = getlistaddr(listdir);
+	listname = genlistname(listaddr);
+	listfqdn = genlistfqdn(listaddr);
+
+	fromaddr = concatstr(3, listname, "+bounces-help@", listfqdn);
+
+	queuefilename = prepstdreply(listdir, "sub-subscribed", "$helpaddr$",
+				     subaddr, NULL, 0, NULL);
+	MY_ASSERT(queuefilename);
+
+	myfree(listaddr);
+	myfree(listname);
+	myfree(listfqdn);
+	
+	execlp(mlmmjsend, mlmmjsend,
+				"-l", "1",
+				"-T", subaddr,
+				"-F", fromaddr,
+				"-m", queuefilename, (char *)NULL);
+	log_error(LOG_ARGS, "execlp() of '%s' failed", mlmmjsend);
+
+	exit(EXIT_FAILURE);
+}
+
 int main(int argc, char **argv)
 {
 	char *listaddr, *listdir = NULL, *address = NULL, *subfilename = NULL;
@@ -430,6 +459,8 @@ int main(int argc, char **argv)
 		close(sublockfd);
 		unlink(sublockname);
 		myfree(sublockname);
+
+		generate_subscribed(listdir, address, mlmmjsend);
 		
 		return EXIT_SUCCESS;
 	}
