@@ -1,6 +1,6 @@
 /* Copyright (C) 2002, 2003 Mads Martin Joergensen <mmj at mmj.dk>
  *
- * $Id: mlmmj-unsub.c,v 1.36 2005/02/14 22:56:45 mmj Exp $
+ * $Id: mlmmj-unsub.c,v 1.39 2005/05/09 13:21:27 mmj Exp $
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -284,6 +284,36 @@ static void print_help(const char *prg)
 	exit(EXIT_SUCCESS);
 }
 
+void generate_notsubscribed(const char *listdir, const char *subaddr,
+		const char *mlmmjsend)
+{
+	char *queuefilename, *fromaddr, *listname, *listfqdn, *listaddr;
+
+	listaddr = getlistaddr(listdir);
+	listname = genlistname(listaddr);
+	listfqdn = genlistfqdn(listaddr);
+
+	fromaddr = concatstr(3, listname, "+bounces-help@", listfqdn);
+
+	queuefilename = prepstdreply(listdir, "unsub-notsubscribed",
+				     "$helpaddr$", subaddr, NULL, 0, NULL);
+	MY_ASSERT(queuefilename);
+
+	myfree(listaddr);
+	myfree(listname);
+	myfree(listfqdn);
+
+	execlp(mlmmjsend, mlmmjsend,
+				"-l", "1",
+				"-T", subaddr,
+				"-F", fromaddr,
+				"-m", queuefilename, (char *)NULL);
+
+	log_error(LOG_ARGS, "execlp() of '%s' failed", mlmmjsend);
+	exit(EXIT_FAILURE);
+}
+
+
 int main(int argc, char **argv)
 {
 	int subread, subwrite, rlock, wlock, opt, unsubres, status, nomail = 0;
@@ -400,9 +430,12 @@ int main(int argc, char **argv)
 	}
 
 	if(is_subbed_in(subddirname, address)) {
-		/* Address is not subscribed, so exit silently */
+		/* Address is not subscribed */
 		myfree(subddirname);
 		myfree(listaddr);
+
+		generate_notsubscribed(listdir, address, mlmmjsend);
+
 		exit(EXIT_SUCCESS);
 	}
 
